package org.cs.snc;

import java.awt.*;
import java.awt.image.BufferedImage;

public class FitnessCalc {

    private static BufferedImage solution = null;
    private static boolean backgroundIsLighter = false;


    /* ******************************************************************************************* */
    // Function Name: setSolution                                                                  //
    // Return Type: void                                                                           //
    // Parameters: java.awt.image.BufferedImage                                                    //
    // Description: This function is called from the RunSceneController.runAlgorithm function.     //
    //              The solution variable is set to the BufferedImage that is given as the         //
    //              parameter to the function. This function also determines if the solution image //
    //              is, on average, darker or lighter. If it is lighter, the background is set to  //
    //              white, else the background is set as black.                                    //
    /* ******************************************************************************************* */
    public static void setSolution(BufferedImage theSolution) {
        solution = theSolution;

        int lighter = 0;
        for(int i = 0; i < RunSceneController.imageWidth; i++) {
            for(int j = 0; j < RunSceneController.imageHeight; j++) {
                int color_int = solution.getRGB(i, j);
                Color color = new Color(color_int);

                int gray = (color.getRed() + color.getGreen() + color.getBlue()) / 3;
                if(gray > 127) {
                    lighter++;
                }
            }
        }

        int halfOfPixels = (RunSceneController.imageWidth * RunSceneController.imageHeight) / 2;
        if(lighter > halfOfPixels) {
            backgroundIsLighter = true;
        }
    }


    public static boolean getIsLighter() {
        return backgroundIsLighter;
    }


    /* ******************************************************************************************* */
    // Function Name: getFitness                                                                   //
    // Return Type: long                                                                           //
    // Parameters: int[]                                                                           //
    // Description: The function is called from an Individual to calculate the fitness of the      //
    //              Individual. The function is passed the integer array that contains the data of //
    //              the Individual. The function calls draw, which draws the Individual to a       //
    //              BufferedImage. That generated BufferedImage is then compared against the       //
    //              solution BufferedImage pixel-by-pixel. Each pixel is given a fitness, and then //
    //              the pixel's fitness is added to a total fitness. Because the fitness is        //
    //              calculated this way, a lower fitness it better.                                //
    /* ******************************************************************************************* */
    public static long getFitness(int[] individual) {
        BufferedImage image = draw(individual);
        long fitness = 0;
        for(int i = 0; i < RunSceneController.imageWidth; i++) {
            for(int j = 0; j < RunSceneController.imageHeight; j++) {
                int solution_int = solution.getRGB(i, j);
                int generated_int = image.getRGB(i, j);
                int error = getColorError(solution_int, generated_int);
                fitness += error;
            }
        }
        return fitness;
    }


    public static BufferedImage draw(int[] individual) {
        BufferedImage image = new BufferedImage(RunSceneController.imageWidth, RunSceneController.imageHeight, BufferedImage.TYPE_3BYTE_BGR);
        Graphics2D g = image.createGraphics();

        if(backgroundIsLighter) {
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, RunSceneController.imageWidth, RunSceneController.imageHeight);
        }

        int geneSize = (RunSceneController.numOfVertices * 2) + 1;
        Polygon p = new Polygon();
        for(int i = 0; i < (geneSize * RunSceneController.numOfPolygonsPerIndividual); i+=geneSize) {
            int x, y;
            for(int j = 0; j < geneSize - 1; j+=2) {
                x = individual[i + j];
                y = individual[i + j + 1];
                p.addPoint(x, y);
            }

            g.setColor(new Color(individual[i + geneSize - 1], true));
            g.fillPolygon(p);
            p.reset();
        }
        g.dispose();

        return image;
    }


    private static int getColorError(int c1, int c2) {
        Color color1 = new Color(c1);
        Color color2 = new Color(c2);

        int red = color1.getRed() - color2.getRed();
        int green = color1.getGreen() - color2.getGreen();
        int blue = color1.getBlue() - color2.getBlue();

        return (red * red) + (green * green) + (blue * blue);
    }
}
