package org.cs.snc;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.image.Image;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.ImagePattern;
import javafx.scene.shape.Circle;
import javafx.stage.Stage;

public class InfoSceneController implements Initializable {

    // @FXML tag is required for any JavaFX GUI element that is
    // initialized at FXML load time instead of in the Java controller class
    @FXML private AnchorPane anchorPane;
    @FXML private Circle circle;

    /* ******************************************************************************************* */
    // Function Name: initialize                                                                   //
    // Return Type: void                                                                           //
    // Parameters: URL, ResourceBundle                                                             //
    // Description: The initialize function is required to override when implementing the JavaFX   //
    //              Initializable class. A Java class typically implements the Initializable class //
    //              if it is the controller of an fxml file. The initialize function is called     //
    //              once the contents of the fxml file that the Java class controls have been      //
    //              loaded. In this function, an image is taken from the resource folder and set   //
    //              as the image of the Circle object.                                             //
    /* ******************************************************************************************* */
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        Image image = new Image(App.class.getResourceAsStream("images/dna.png"));
        circle.setFill(new ImagePattern(image));
    }


    /* ******************************************************************************************* */
    // Function Name: loadScene2                                                                   //
    // Return Type: void                                                                           //
    // Parameters: javafx.event.ActionEvent                                                        //
    // Description: This function is linked to a button via InfoSceneView.fxml. When clicked,      //
    //              the next scene is loaded to the window with FXMLLoader.load and the window is  //
    //              resized.                                                                       //
    /* ******************************************************************************************* */
    @FXML
    private void loadScene2(ActionEvent actionEvent) throws IOException {
        Stage window = (Stage)((Button)actionEvent.getSource()).getScene().getWindow();

        AnchorPane pane = FXMLLoader.load(App.class.getResource("fxml/ConfigSceneView.fxml"));
        anchorPane.getChildren().setAll(pane);

        window.setWidth(1300);
        window.setHeight(800);
        window.setResizable(false);
        window.centerOnScreen();
    }
}
